@@ -11,7 +11,7 @@ static const char *copyright[] = {
 	"",
 	" Author(s):",
 	"    Robert Stiles, KK5VD, Copyright (C) 2013, 2014, 2015",
-	"    Dave Freese, W1HKJ, Copyright (C) 2012, 2013, 2014, 2015",
+	"    Dave Freese, W1HKJ, Copyright (C) 2012-2015, 2023",
 	"",
 	" This is free software; you can redistribute it and/or modify",
 	" it under the terms of the GNU General Public License as published by",
@@ -58,10 +58,13 @@ static const char *copyright[] = {
 #include <FL/Fl_Help_Dialog.H>
 #include <FL/Fl_Menu_Item.H>
 #include <FL/Fl_File_Icon.H>
+#include <FL/filename.H>
+#include <FL/Fl_Native_File_Chooser.H>
 
 #include "flamp.h"
 #include "amp.h"
 #include "flamp_dialog.h"
+#include "dialogs.h"
 
 #include "debug.h"
 #include "util.h"
@@ -212,6 +215,7 @@ pthread_mutex_t mutex_file_io = PTHREAD_MUTEX_INITIALIZER;
 
 int xmlrpc_errno = 0;
 int file_io_errno = 0;
+int reading_relay_data = 0;
 
 
 std::string NBEMS_dir = "";
@@ -222,6 +226,7 @@ std::string flamp_rcv_dir = "";
 std::string flamp_xmt_dir = "";
 std::string flamp_script_dir = "";
 std::string flamp_script_default_dir = "";
+std::string flamp_relay_dir = "";
 std::string buffer = "";
 
 std::string cmd_fname = "";
@@ -252,7 +257,7 @@ bool rx_complete = false;
 bool isbinary(std::string s)
 {
 	for (size_t n = 0; n < s.length(); n++)
-		if ((s[n] & 0x80) == 0x80) return true;
+	if ((s[n] & 0x80) == 0x80) return true;
 	return false;
 }
 
@@ -673,11 +678,12 @@ void checkdirectories(void)
 	};
 
 	DIRS NBEMS_dirs[] = {
-		{ NBEMS_dir,      0,        0 },
-		{ flampHomeDir,     "FLAMP",	0 },
+		{ NBEMS_dir,        0,               0 },
+		{ flampHomeDir,     "FLAMP",	     0 },
 		{ flamp_rcv_dir,    "FLAMP/rx",      0 },
 		{ flamp_xmt_dir,    "FLAMP/tx",      0 },
-		{ flamp_script_dir, "FLAMP/scripts",      0 },
+		{ flamp_script_dir, "FLAMP/scripts", 0 },
+		{ flamp_relay_dir,  "FLAMP/relay",   0 }
 	};
 
 	int r;
@@ -688,8 +694,8 @@ void checkdirectories(void)
 
 		if ((r = mkdir(NBEMS_dirs[i].dir.c_str(), 0777)) == -1 && errno != EEXIST) {
 			std::cerr << _("Could not make directory") << ' ' << NBEMS_dirs[i].dir
-			     << ": " << strerror(errno) << '\n';
-			exit(EXIT_FAILURE);
+			<< ": " << strerror(errno) << '\n';
+			clean_shutdown();
 		}
 		else if (r == 0 && NBEMS_dirs[i].new_dir_func)
 			NBEMS_dirs[i].new_dir_func();
@@ -707,7 +713,6 @@ void addfile(ScriptParsing *sp, SCRIPT_COMMANDS *sc)
 	bool useCompression = sp->comp();
 	bool proto = sp->proto();
 	std::string desc;
-	//int block_size = sp->blocks();
 
 	xmtfname.assign(sp->file());
 	xmt_fname = xmtfname;
@@ -830,7 +835,7 @@ void addfile(ScriptParsing *sp, SCRIPT_COMMANDS *sc)
  *
  ***********************************************************/
 void addfile(std::string xmtfname, void *rx, bool useCompression, \
-             char *desc = (char *)0, char *callto = (char *)0)
+			 char *desc = (char *)0, char *callto = (char *)0)
 {
 	xmt_fname = xmtfname;
 	std::string xmt_fname2 = xmtfname;
@@ -1198,7 +1203,7 @@ void auto_load_tx_queue_from_list(void)
 	size = strnlen(srcpath, FILENAME_MAX);
 
 	for(i = 0; i < size; i++)
-		if(srcpath[i] == '\\') srcpath[i] = '/';
+	if(srcpath[i] == '\\') srcpath[i] = '/';
 
 	fd = fopen(srcpath, "r");
 
@@ -1348,7 +1353,7 @@ void auto_load_tx_queue(void)
 /** ********************************************************
  *
  ***********************************************************/
-void readfile()
+void readfile(void)
 {
 	std::string xmtfname;
 	xmtfname.clear();
@@ -1374,13 +1379,16 @@ void update_rx_missing_blocks(void)
 	amp->rx_relay_blocks(tmp);
 }
 
+extern void redraw_rx_panel(void *);
+
 /** ********************************************************
  *
  ***********************************************************/
-void show_rx_amp()
+void show_rx_amp(void)
 {
 	cAmp * amp = rx_amp.get_amp();
 	if (!amp) return;
+	std::string time_size_info;
 
 	txt_rx_filename->value(amp->get_rx_fname().c_str());
 	txt_rx_datetime->value(amp->rx_time_stamp().c_str());
@@ -1392,6 +1400,10 @@ void show_rx_amp()
 	txt_rx_missing_blocks->value(amp->rx_missing().c_str());
 	txt_relay_selected_blocks->value(amp->rx_relay_blocks().c_str());
 	rx_progress->set(amp->rx_blocks(), amp->rx_nblocks());
+	cnt_repeat_relay_data->value(progStatus.repeat_relay_data);
+	cnt_repeat_relay_header->value(progStatus.repeat_relay_header);
+	estimate_relay();
+
 	if (amp->rx_completed() && txt_rx_output->buffer()->length() == 0) {
 		std::string data = amp->rx_recvd_string();
 		decompress_maybe(data);
@@ -1400,12 +1412,14 @@ void show_rx_amp()
 		else
 			txt_rx_output->addstr(data.c_str());
 	}
+	
+	Fl::awake(redraw_rx_panel, (void *)0);
 }
 
 /** ********************************************************
  *
  ***********************************************************/
-void clear_rx_amp()
+void clear_rx_amp(void)
 {
 	txt_rx_filename->value("");
 	txt_rx_datetime->value("");
@@ -1469,8 +1483,10 @@ static const char *cancel = "Cancel";
 static const char *yes = "Yes";
 static const char *cont = "You are about to transmit! Continue?";
 
-void send_missing_report()
+void send_missing_report(void)
 {
+	if (fldigi_state != FLDIGI_ONLINE) return;
+		
 	std::string fname = txt_rx_filename->value();
 	if (fname.empty()) return;
 
@@ -1488,8 +1504,6 @@ void send_missing_report()
 	}
 
 	if(progStatus.use_tx_on_report) {
-		if (!bConnected) connect_to_fldigi(0);
-		if (!bConnected) return;
 		int results = fl_choice("%s", cancel, yes, (const char *)0, cont);
 		if(results < 1) return;
 		transfer(report);
@@ -1502,7 +1516,7 @@ void send_missing_report()
 /** ********************************************************
  *
  ***********************************************************/
-void recv_missing_report()
+void recv_missing_report(void)
 {
 	cAmp *amp = 0;
 	size_t count = (size_t) tx_amp.size();
@@ -1521,7 +1535,7 @@ void recv_missing_report()
 /** ********************************************************
  *
  ***********************************************************/
-void relay_missing_report()
+void relay_missing_report(void)
 {
 	cAmp *camp = 0;
 	size_t count = (size_t) rx_amp.size();
@@ -1546,13 +1560,10 @@ void relay_missing_report()
 /** ********************************************************
  *
  ***********************************************************/
-void send_relay_data()
+void send_relay_data(void)
 {
-	if(transmitting)
-		return;
-
-	if (!bConnected) connect_to_fldigi(0);
-	if (!bConnected) return;
+	if (fldigi_state != FLDIGI_ONLINE) return;
+	if(transmitting) return;
 
 	cAmp *amp = rx_amp.get_amp();
 
@@ -1582,7 +1593,7 @@ void send_relay_data()
 		relay_data.header = amp->xmt_vector_header();
 		relay_data.data = amp->xmt_vector_data();
 	} else {
-		relay_data.serial_data = amp->tx_relay_string(progStatus.my_call, missing_blocks);
+		relay_data.serial_data = amp->tx_relay_string(progStatus.my_call, missing_blocks, false);
 	}
 
 	results = fl_choice("%s", cancel, yes, (const char *)0, cont);
@@ -1653,6 +1664,15 @@ void tx_removefile(bool all)
 	}
 }
 
+/** ********************************************************
+ *
+ ***********************************************************/
+void save_all_rx_relay_files(void)
+{
+	if(rx_amp.size())
+		rx_amp.save_all_relay();
+}
+
 /** ********************************************************
  *
  ***********************************************************/
@@ -1740,13 +1760,18 @@ void auto_rx_save_file(cAmp *_amp)
  ***********************************************************/
 void writefile(int xfrFlag)
 {
+	std::string msg;
 	cAmp * amp = rx_amp.get_amp();
 	if (!amp) return;
 
 	size_t fsize = amp->rx_size();
 
-	if(xfrFlag && !amp->rx_completed()) {
-		fl_alert2("Only completed files can be transfered");
+	if(!amp->rx_completed()) {
+		if(xfrFlag)
+			msg.assign("transfered");
+		else
+			msg.assign("saved");
+		fl_alert2("Only completed files can be %s", msg.c_str());
 		return;
 	}
 
@@ -1997,23 +2022,18 @@ int receive_data_stream(void *ptr)
 	int n = 0;
 	int size = sizeof(buffer) - 1;
 
-	if (!bConnected) {
-		connect_to_fldigi(0);
-		if (!bConnected) {
-			cQue->sleep(2, 0);
-			return 0;
-		}
-	}
-	else {
+	if (fldigi_state != FLDIGI_ONLINE) return 0;
+	
+	if(!reading_relay_data)
 		n = rx_fldigi(buffer, size);
-		if(n < 1) {
-			cQue->milliSleep(50);
-			return 0;
-		}
 
-		cQue->addToQueueNullFiltered(buffer, n);
+	if(n < 1) {
+		cQue->milliSleep(50);
+		return 0;
 	}
 
+	cQue->addToQueueNullFiltered(buffer, n);
+	
 	return n;
 }
 
@@ -2162,6 +2182,161 @@ void process_missing_stream(void)
 	}
 }
 
+/** ********************************************************
+ *
+ ***********************************************************/
+void process_relay_files(void)
+{
+	std::vector<std::string> files;
+	std::string file;
+	int count = 0;
+    bool tmp = progStatus.auto_rx_save;
+
+
+	count = FSEL::select_multiple(_("Select Relay File(s)"), "*.*",
+											files, flamp_relay_dir.c_str());
+
+	reading_relay_data = true; // Temp disable FLDIGI input
+
+	progStatus.auto_rx_save = false; // Disable Auto Save RX files
+
+	for (int n = 0; n < count; n++ ) {
+		file.assign(files[n]);
+		process_data_file(file);
+	}
+
+	progStatus.auto_rx_save = tmp;
+
+	reading_relay_data = false;
+}
+
+/** ********************************************************
+ *
+ ***********************************************************/
+void process_data_file(std::string file)
+{
+	std::string retbuff;
+	std::string temp_buff;
+
+	char tag[32];
+	char crc[5];
+	char hash[5];
+	int len;
+	size_t conv;
+	cAmp *tmp = 0;
+	size_t i  = 0;
+	size_t count = rx_amp.size();
+	size_t mark_pos   = 0;
+	signed long end_count  = 0;
+	size_t length     = 0;
+	size_t pos        = 0;
+
+	std::ifstream relay_file (file.c_str());
+
+	if(relay_file) {
+
+		relay_file.seekg (0, relay_file.end);
+		length = relay_file.tellg();
+		relay_file.seekg (0, relay_file.beg);
+
+		char * buffer = new char [length];
+
+		if(buffer) {
+			relay_file.read (buffer, length);
+			relay_file.close();
+		} else {
+			return;
+		}
+
+		while(pos < length) {
+
+			if(buffer[pos] == '<')
+				goto check_data;
+			else {
+				pos++;
+				continue;
+			}
+
+		check_data:
+
+			conv = sscanf(&buffer[pos], "<%s %d %4s>{%4s", tag, &len, crc, hash);
+
+			if (conv != 4) {
+				pos++;
+				continue;
+			}
+
+			end_count = ((signed long) length) - ((signed long) pos);
+
+			if(end_count > CNT_BLOCK_SIZE_MAXIMUM)
+				end_count = CNT_BLOCK_SIZE_MAXIMUM;
+
+			if(end_count < len) break; // Not enough data to process
+
+			temp_buff.assign((const char *) &buffer[pos], (size_t) end_count);
+
+			mark_pos = temp_buff.find(">{");
+
+			retbuff.assign(temp_buff.substr(0, len + mark_pos + 1));
+
+			pos += retbuff.size();
+
+			cAmp *existing = 0;
+
+			for (i = 0; i < count; i++) {
+				tmp = rx_amp.index2amp(i + 1);
+				if (tmp->hash(hash)) {
+					existing = tmp;
+					break;
+				}
+			}
+
+			if (!existing) { // a new rx process
+
+				std::string temp_fname;
+				temp_fname.assign("Unassigned");
+
+				cAmp *nu = new cAmp();
+				nu->amp_type(RX_AMP);
+				nu->rx_fname(temp_fname);
+				nu->rx_hash(hash);
+				nu->rx_to_tx_hash(); // For relay ops
+				nu->rx_append(retbuff);
+				nu->rx_parse_buffer();
+				rx_amp.add(nu);
+				std::string s;
+				s.assign("@f").append(nu->rx_sz_percent()).append("\t").append(nu->rx_hash()).append("\t");
+				s.append(nu->get_rx_fname());
+				rx_queue->add(s.c_str());
+				rx_queue->select(rx_queue->size());
+				rx_amp.set(nu);
+				clear_rx_amp();
+				show_rx_amp();
+				LOG_DEBUG("New Amp instance: %s", nu->get_rx_fname().c_str());
+				count = rx_amp.size();
+
+			} else {
+
+				std::string bline;
+				if (!existing->rx_completed()) {
+					existing->rx_append(retbuff);
+					existing->rx_parse_buffer();
+					bline.assign("@f").append(existing->rx_sz_percent()).append("\t").append(existing->rx_hash()).append("\t").append(existing->get_rx_fname());
+					rx_queue->text(i+1, bline.c_str());
+				} else {
+					if(progStatus.auto_rx_save && !existing->file_saved()) {
+						auto_rx_save_file(existing);
+					}
+				}
+			}
+		}
+
+		delete[] buffer;
+	}
+
+	if (count != 0)
+		show_rx_amp();
+}
 
 /** ********************************************************
  *
@@ -2286,61 +2461,40 @@ void receive_remove_from_queue(bool all)
 	}
 }
 
-#if 0
 /** ********************************************************
  *
  ***********************************************************/
-void estimate(cAmp *amp, bool visable) {
+ std::string calc_string_tx_time(std::string string_data)
+ {
+ 	char sz_xfr_size[100];
+ 	float xfr_time = 0;
+ 	float oh = 0;
+ 	int transfer_size = 0;
+	static std::string ret_str;
 
-	static char sz_xfr_size[50];
-	float xfr_time = 0;
-	float oh = 0;
-	int transfer_size;
-	int n = 0;
+	memset(sz_xfr_size, 0, sizeof(sz_xfr_size));
 
-	if(tx_amp.size() < 1)
-		return;
+	xfr_time = seconds_from_string(cbo_modes->value(), string_data, &oh);
+	xfr_time += oh;
+ 	transfer_size = string_data.size();
 
-	if(!amp) {
-		n = tx_queue->value();
-		if(n < 1) return;
-		amp = tx_amp.index2amp(n);
-		if(!amp) return;
-	}
+	if (xfr_time < 60)
+		snprintf(sz_xfr_size, sizeof(sz_xfr_size), "%d bytes / %d secs",
+				 transfer_size, (int)(xfr_time + 0.5));
+	else
+		snprintf(sz_xfr_size, sizeof(sz_xfr_size), "%d bytes / %d m %d s",
+				 transfer_size,
+				 (int)(xfr_time / 60), ((int)xfr_time) % 60);
 
-	amp->amp_update();
-	std::string xmtstr = amp->tx_string(" K\n");
+ 	ret_str.assign(sz_xfr_size);
 
-	if(visable) {
-
-		transfer_size = xmtstr.length();
-
-		if (transfer_size == 0) {
-			txt_transfer_size_time->value("");
-			return;
-		}
-
-
-		xfr_time = seconds_from_string(cbo_modes->value(), xmtstr, &oh);
-		xfr_time += oh;
-
-		if (xfr_time < 60)
-			snprintf(sz_xfr_size, sizeof(sz_xfr_size), "%d bytes / %d secs",
-					 transfer_size, (int)(xfr_time + 0.5));
-		else
-			snprintf(sz_xfr_size, sizeof(sz_xfr_size), "%d bytes / %d m %d s",
-					 transfer_size,
-					 (int)(xfr_time / 60), ((int)xfr_time) % 60);
-
-		txt_transfer_size_time->value(sz_xfr_size);
-		txt_tx_numblocks->value(amp->xmt_numblocks().c_str());
-	}
+	return ret_str;
 }
-#else
+
 /** ********************************************************
  *
  ***********************************************************/
-void estimate(cAmp *amp, bool visable)
+void estimate(cAmp *amp, bool visible)
 {
 	int n = 0;
 
@@ -2356,13 +2510,39 @@ void estimate(cAmp *amp, bool visable)
 
 	amp->amp_update();
 
-	if(visable) {
+	if(visible) {
 		txt_transfer_size_time->value(amp->estimate().c_str());
 		txt_tx_numblocks->value(amp->xmt_numblocks().c_str());
 	}
 }
 
-#endif
+/** ********************************************************
+ *
+ ***********************************************************/
+void estimate_relay(void)
+{
+	cAmp *amp = (cAmp *) 0;
+	int n = 0;
+	std::string data;
+
+	if(rx_amp.size() < 1)
+		return;
+
+	n = rx_queue->value();
+	if(n < 1) return;
+	amp = rx_amp.index2amp(n);
+	if(!amp) return;
+
+	data.assign(txt_relay_selected_blocks->value());
+	amp->rx_relay_blocks(data);
+	data = amp->rx_relay_blocks();
+
+	data = amp->tx_relay_string(progStatus.my_call, data, false);
+	data = calc_string_tx_time(data);
+
+	txt_transfer_relay_size_time->value(data.c_str());
+}
+
 
 /** ********************************************************
  *
@@ -2461,8 +2641,7 @@ void estimate_bc(void) {
 /** ********************************************************
  *
  ***********************************************************/
-
-void close_ops()
+void close_ops(void)
 {
 	progStatus.saveLastState();
 
@@ -2480,38 +2659,52 @@ void close_ops()
 	debug::stop();
 
 	if(cQue) delete cQue;
+	if(s_modems) delete s_modems;
+
 }
 
-void cb_exit()
+/** ********************************************************
+ *
+ ***********************************************************/
+void clean_shutdown()
 {
 	if(transmitting) {
 		transmit_stop = true;
 	}
-	exit_watch_dog = true;
-	close_ops();
+//	exit_watch_dog = true;
 
+	close_ops();
 	exit_server();
+	TOD_close();
 
 	exit(0);
 }
 
+void cb_exit(void)
+{
+	clean_shutdown();
+}
+
 /** ********************************************************
  *  test for running fldigi made immediately after starting
  *  the xmlrpc thread
  ***********************************************************/
 
-void test_for_fldigi()
-{	std::string test = get_io_mode();
+void test_for_fldigi(void)
+{
+	return;
+// remaining code in function never called !
+	std::string test = get_io_mode();
 	if (test == "NIL" || test.empty()) {
-		fl_alert2(_("Start fldigi before flamp!"));
+//		fl_alert2(_("Start fldigi before flamp!"));
 		if (tcpip) {
 			tcpip->close();
 			delete tcpip;
 			delete localaddr;
 		}
-		debug::stop();
-		if(cQue) delete cQue;
-		exit(0);
+//		debug::stop();
+//		if(cQue) delete cQue;
+//		clean_shutdown();
 	}
 }
 
@@ -2685,44 +2878,44 @@ int main (int argc, char *argv[])
 
 	NBEMS_dir.clear();
 	{
-		std::string appname = argv[0];
-		std::string appdir = argv[0];
-		size_t p;
-		char dirbuf[FL_PATH_MAX + 1];
+	std::string appname = argv[0];
+	std::string appdir = argv[0];
+	size_t p;
+	char dirbuf[FL_PATH_MAX + 1];
 #ifdef __WOE32__
-		p = appdir.find("FL_APPS\\");
-		if (p == std::string::npos) p = appdir.find("FL_APPS/");
-		if (p == std::string::npos) {
-			char dirbuf[FL_PATH_MAX + 1];
-			fl_filename_expand(dirbuf, sizeof(dirbuf) -1, "$USERPROFILE/");
-			NBEMS_dir.assign(dirbuf);
-		} else
-			NBEMS_dir.assign(appdir.substr(0, p + 8));
-		NBEMS_dir.append("NBEMS.files/");
+	p = appdir.find("FL_APPS\\");
+	if (p == std::string::npos) p = appdir.find("FL_APPS/");
+	if (p == std::string::npos) {
+		char dirbuf[FL_PATH_MAX + 1];
+		fl_filename_expand(dirbuf, sizeof(dirbuf) -1, "$USERPROFILE/");
+		NBEMS_dir.assign(dirbuf);
+	} else
+		NBEMS_dir.assign(appdir.substr(0, p + 8));
+	NBEMS_dir.append("NBEMS.files/");
 #else
-		fl_filename_absolute(dirbuf, sizeof(dirbuf), argv[0]);
-		appdir.assign(dirbuf);
-		p = appdir.rfind("flamp");
-		if (p != std::string::npos)
-			appdir.erase(p);
-		p = appdir.find("FL_APPS/");
-		if (p != std::string::npos)
-			NBEMS_dir.assign(appdir.substr(0, p + 8));
-		else {
-			fl_filename_expand(dirbuf, FL_PATH_MAX, "$HOME/");
-			NBEMS_dir = dirbuf;
-		}
+	fl_filename_absolute(dirbuf, sizeof(dirbuf), argv[0]);
+	appdir.assign(dirbuf);
+	p = appdir.rfind("flamp");
+	if (p != std::string::npos)
+		appdir.erase(p);
+	p = appdir.find("FL_APPS/");
+	if (p != std::string::npos)
+		NBEMS_dir.assign(appdir.substr(0, p + 8));
+	else {
+		fl_filename_expand(dirbuf, FL_PATH_MAX, "$HOME/");
+		NBEMS_dir = dirbuf;
+	}
 
-		DIR *isdir = 0;
-		std::string test_dir;
-		test_dir.assign(NBEMS_dir).append("NBEMS.files/");
-		isdir = opendir(test_dir.c_str());
-		if (isdir) {
-			NBEMS_dir = test_dir;
-			closedir(isdir);
-		} else {
-			NBEMS_dir.append(".nbems/");
-		}
+	DIR *isdir = 0;
+	std::string test_dir;
+	test_dir.assign(NBEMS_dir).append("NBEMS.files/");
+	isdir = opendir(test_dir.c_str());
+	if (isdir) {
+		NBEMS_dir = test_dir;
+		closedir(isdir);
+	} else {
+		NBEMS_dir.append(".nbems/");
+	}
 
 #endif
 	}
@@ -2764,7 +2957,7 @@ int main (int argc, char *argv[])
 	main_window->icon((char *)flamp_icon_pixmap);
 	main_window->show(argc, argv);
 
-// read in the current window hints, then modify them to allow icon transparency
+	// read in the current window hints, then modify them to allow icon transparency
 	Pixmap mask = -1; // create pixmaps to hold the icon image and its mask
 	XWMHints* hints = XGetWMHints(fl_display, fl_xid(main_window));
 	hints->flags |= IconMaskHint; // ensure transparency mask is enabled for the XPM icon
@@ -2796,18 +2989,13 @@ int main (int argc, char *argv[])
 		port.assign(progStatus.socket_port);
 
 	localaddr = new Address(addr.c_str(), port.c_str());
-	if (!localaddr) exit(EXIT_FAILURE);
-	tcpip = new Socket (*localaddr);
-	tcpip->set_timeout(0.01);
-	connect_to_fldigi(0);
+	if (!localaddr) clean_shutdown();
 
-	open_xmlrpc();
 	xmlrpc_thread = new pthread_t;
 	if (pthread_create(xmlrpc_thread, NULL, xmlrpc_loop, NULL)) {
 		perror("pthread_create: xmlrpc");
-		exit(EXIT_FAILURE);
+		clean_shutdown();
 	}
-	test_for_fldigi();
 
 	try {
 		cQue = new TagSearch(receive_data_stream, process_que);
@@ -2825,26 +3013,23 @@ int main (int argc, char *argv[])
 	cnt_repeat_nbr->value(progStatus.repeatNN);
 	cnt_repeat_header->value(progStatus.repeat_header);
 
+	cnt_repeat_relay_data->value(progStatus.repeat_relay_data);
+	cnt_repeat_relay_header->value(progStatus.repeat_relay_header);
+	
 	tx_buffer.clear();
 	rx_buffer.clear();
 
-	watch_dog_seconds = time(0);
-	watch_dog_thread = new pthread_t;
-	if (pthread_create(watch_dog_thread, NULL, watch_dog_loop, NULL)) {
-		perror("pthread_create: ztimer watch dog not started");
-	}
+	TOD_init();
 
 	start_server(61960);
 
-	Fl::add_timeout(0, ztimer, (void *)true);
-
 	if(progStatus.auto_load_queue) {
 		auto_load_tx_queue();
 	}
 
 	unproto_widgets(0);
 
-	Fl::add_timeout(0.5, check_io_mode);
+	Fl::add_timeout(1.0, check_io_mode);
 
 	return Fl::run();
 }
@@ -2854,32 +3039,41 @@ int main (int argc, char *argv[])
  ***********************************************************/
 void check_io_mode(void *v)
 {
+	check_call_and_id(v);
+
 	// Check to see what io mode FLDIGI is in.
 	std::string io_mode = get_io_mode();
+
+	if (io_mode == "NIL") return;
+
 	int flag = 0;
 	if(!io_mode.empty()) {
-		flag = strncmp(io_mode.c_str(), "ARQ", 3);
-		if(flag != 0) {
-			flag = fl_choice2(_("KISS interface active! Switch FLDIGI to ARQ?"),
-							  _("No"), _("Yes"), NULL);
-			if(flag == 1)
-				enable_arq();
-		}
+		if(io_mode == "ARQ") return;
+		
+		flag = fl_choice2(_("KISS interface active! Switch FLDIGI to ARQ?"),
+				_("No"), _("Yes"), NULL);
+				
+		if(flag == 1)
+			enable_arq();
 	}
-
-	check_call_and_id(v);
 }
 
+extern Fl_Group *tab_pointers[];
 
 /** ********************************************************
  *
  ***********************************************************/
 void check_call_and_id(void *v)
-{
-	if(progStatus.my_call.empty() || progStatus.my_info.empty()) {
-		if(tabs && Config_tab) {
-			tabs->value(Config_tab);
-			fl_choice2(_("Update Callsign and Info"), _("Okay"), NULL, NULL);
+{	
+	if(tabs) {
+		if(progStatus.my_call.empty() || progStatus.my_info.empty()) {
+			if(Config_tab) {
+				tabs->value(Config_tab);
+				fl_choice2(_("Update Callsign and Info"), _("Okay"), NULL, NULL);
+			}
+		} else {
+			if(tab_pointers[progStatus.tab_saved_state])
+				tabs->value(tab_pointers[progStatus.tab_saved_state]);    // User Selected
 		}
 	}
 }
@@ -2912,9 +3106,9 @@ void open_url(const char* url)
 			unsetenv("MALLOC_PERTURB_");
 #  endif
 			for (size_t i = 0; i < sizeof(browsers)/sizeof(browsers[0]); i++)
-				if (browsers[i])
-					execlp(browsers[i], browsers[i], url, (char*)0);
-			exit(EXIT_FAILURE);
+			if (browsers[i])
+				execlp(browsers[i], browsers[i], url, (char*)0);
+			clean_shutdown();
 		case -1:
 			fl_alert2(_("Could not run a web browser:\n%s\n\n"
 						"Open this URL manually:\n%s"),
@@ -2929,7 +3123,7 @@ void open_url(const char* url)
 /** ********************************************************
  *
  ***********************************************************/
-void show_help()
+void show_help(void)
 {
 	open_url("http://www.w1hkj.com/flamp-help/index.html");
 }
@@ -2937,7 +3131,7 @@ void show_help()
 /** ********************************************************
  *
  ***********************************************************/
-void cb_folders()
+void cb_folders(void)
 {
 	open_url(flampHomeDir.c_str());
 }
@@ -2991,10 +3185,10 @@ void drop_file_changed(void)
 	int valid = 0;
 
 	char *cPtr = (char *)0;
-//	char *cFileName = (char *)0;
+	//	char *cFileName = (char *)0;
 	const char *cBufferEnd = (char *)0;
 
-	drop_file->value("  DnD");
+	drop_file->value("   DnD");
 	drop_file->redraw();
 
 	fname = new char[length + 1];
@@ -3020,7 +3214,7 @@ void drop_file_changed(void)
 	if(valid == 0) return;
 
 	cBufferEnd = &fname[length];
-//	cFileName = 
+	//	cFileName =
 	cPtr = (char *) fname;
 
 	// Skip leading spaces
@@ -3032,8 +3226,8 @@ void drop_file_changed(void)
 
 	// Null terminate control characters
 	while(cPtr < cBufferEnd) {
-	   if(*cPtr < ' ') *cPtr = 0;
-	   cPtr++;
+		if(*cPtr < ' ') *cPtr = 0;
+		cPtr++;
 	}
 
 	addfile(fname, 0, false, 0, 0);
@@ -3044,3 +3238,4 @@ void drop_file_changed(void)
 	}
 }
 
+
