@@ -55,6 +55,7 @@
 #include "flamp.h"
 #include "amp.h"
 #include "flamp_dialog.h"
+#include "dialogs.h"
 
 #include "debug.h"
 #include "util.h"
@@ -96,12 +97,9 @@ static void wait_seconds(int seconds, int millisecond_increments);
 void transfer_in_main_thread(void *ptr)
 {
 	if (!ptr) return;
-
+	if (fldigi_state != FLDIGI_ONLINE) return;
+	
 	std::string *data = (std::string *)ptr;
-
-	if (!bConnected) connect_to_fldigi(0);
-	if (!bConnected) return;
-
 	transfer(*data);
 }
 
@@ -123,7 +121,7 @@ std::string tx_string(cAmp *tx, std::string t_string)
 	if (send_to.empty()) send_to.assign("QST");
 	send_to.append(" DE ").append(progStatus.my_call);
 	for (size_t n = 0; n < send_to.length(); n++)
-		send_to[n] = toupper(send_to[n]);
+	send_to[n] = toupper(send_to[n]);
 
 	tx->my_call(progStatus.my_call);
 	tx->my_info(progStatus.my_info);
@@ -188,8 +186,7 @@ void * create_tx_table(void *ptr)
 	TX_FLDIGI_THREAD *thread_ptr = (TX_FLDIGI_THREAD *)ptr;
 	static bool in_use = false;
 
-	if (!bConnected) connect_to_fldigi(0);
-	if (!bConnected || in_use) {
+	if((fldigi_state != FLDIGI_ONLINE) || in_use) {
 		return run_in_thread_destroy(thread_ptr, 3, 0);
 	}
 
@@ -217,9 +214,9 @@ void * create_tx_table(void *ptr)
 	transmitting = true;
 
 	{
-		Fl::awake(set_button_to_cancel, (void *)0);
-		static int value = TX_ALL_BUTTON;
-		Fl::awake(deactivate_button, (void *) &value);
+	Fl::awake(set_button_to_cancel, (void *)0);
+	static int value = TX_ALL_BUTTON;
+	Fl::awake(deactivate_button, (void *) &value);
 	}
 
 	turn_rsid_off();
@@ -249,6 +246,10 @@ void * create_tx_table(void *ptr)
 
 	if(fd) {
 		LOG_DEBUG("Table Gen filename %s", filename);
+		fprintf(fd, "/**********************************************\n");
+		fprintf(fd, " * Software Generated - Do Not Modify\n");
+		fprintf(fd, " **********************************************/\n");
+
 		fprintf(fd, "typedef struct {\n"
 				"\tchar *mode_name;\n"
 				"\tfloat scale;\n"
@@ -381,10 +382,10 @@ double measure_tx_time(int character_to_send, double *over_head)
 
 	std::string tx_duration;
 
-//	set_xmlrpc_timeout(8.0);
+	//	set_xmlrpc_timeout(8.0);
 	tx_duration = get_char_timing(character_to_send);
 
-//	set_xmlrpc_timeout_default();
+	//	set_xmlrpc_timeout_default();
 
 	if(tx_duration.size() < 1) {
 		LOG_ERROR("No Data returned.");
@@ -418,8 +419,7 @@ double measure_tx_time(int character_to_send, double *over_head)
  ***********************************************************/
 void transmit_relay(RELAY_DATA *rd)
 {
-	if (!bConnected) connect_to_fldigi(0);
-	if (!bConnected) return;
+	if (fldigi_state != FLDIGI_ONLINE)  return;
 
 	if(transmitting) return;
 
@@ -437,8 +437,7 @@ void transmit_relay(RELAY_DATA *rd)
  ***********************************************************/
 void transmit_current()
 {
-	if (!bConnected) connect_to_fldigi(0);
-	if (!bConnected) return;
+	if (fldigi_state != FLDIGI_ONLINE)  return;
 
 	if(transmitting) return;
 
@@ -468,8 +467,7 @@ void transmit_current()
  ***********************************************************/
 void transmit_queued(bool event_driven, bool shift_key_pressed)
 {
-	if (!bConnected) connect_to_fldigi(0);
-	if (!bConnected) return;
+	if (fldigi_state != FLDIGI_ONLINE) return;
 
 	if(transmitting) return;
 
@@ -516,9 +514,9 @@ void * transmit_serial_current(void *ptr)
 	in_use = true;
 
 	{
-		Fl::awake(set_button_to_cancel, (void *)0);
-		static int value = TX_ALL_BUTTON;
-		Fl::awake(deactivate_button, (void *) &value);
+	Fl::awake(set_button_to_cancel, (void *)0);
+	static int value = TX_ALL_BUTTON;
+	Fl::awake(deactivate_button, (void *) &value);
 	}
 
 	int n = tx_queue->value();
@@ -572,7 +570,6 @@ void * transmit_serial_current(void *ptr)
 	tx_time = seconds_from_string(cbo_modes->value(), autosend, &oh);
 	tx_time += oh;
 	tx_time_g = tx_time;
-	tx_time *= 2.0;
 
 	transfer(autosend);
 
@@ -681,7 +678,6 @@ void * transmit_serial_queued(void *ptr)
 	tx_time = seconds_from_string(thread->modem.c_str(), autosend, &oh);
 	tx_time += oh;
 	tx_time_g = tx_time;
-	tx_time *= 1.10;
 
 	wait_for_rx((int) tx_time);
 
@@ -738,8 +734,8 @@ void * transmit_serial_relay(void *ptr)
 	transmitting = true;
 
 	{
-		static char *msg = (char *) CANX_LABEL;
-		Fl::awake(set_relay_button_label, (void *) msg);
+	static char *msg = (char *) CANX_LABEL;
+	Fl::awake(set_relay_button_label, (void *) msg);
 	}
 
 	if(!ptr) return ptr; // This should not happen.
@@ -749,8 +745,8 @@ void * transmit_serial_relay(void *ptr)
 		return run_in_thread_destroy(t_ptr, 3, &in_use);
 
 	{
-		static char *msg = (char *) CANX_LABEL;
-		Fl::awake(set_relay_button_label, (void *) msg);
+	static char *msg = (char *) CANX_LABEL;
+	Fl::awake(set_relay_button_label, (void *) msg);
 	}
 
 	if (progStatus.fldigi_xmt_mode_change)
@@ -782,7 +778,6 @@ void * transmit_serial_relay(void *ptr)
 	tx_time = seconds_from_string(cbo_modes->value(), autosend, &oh);
 	tx_time += oh;
 	tx_time_g = tx_time;
-	tx_time *= 2.0;
 
 	transfer(autosend);
 
@@ -808,8 +803,8 @@ void * transmit_relay_interval(void * ptr)
 	in_use = true;
 
 	{
-		static char *msg = (char *) CANX_LABEL;
-		Fl::awake(set_relay_button_label, (void *) msg);
+	static char *msg = (char *) CANX_LABEL;
+	Fl::awake(set_relay_button_label, (void *) msg);
 	}
 
 	cAmp *tx = (cAmp *)0;
@@ -859,7 +854,7 @@ void * transmit_relay_interval(void * ptr)
 	send_to.clear();
 	send_to.append(" DE ").append(progStatus.my_call);
 	for (size_t n = 0; n < send_to.length(); n++)
-		send_to[n] = toupper(send_to[n]);
+	send_to[n] = toupper(send_to[n]);
 
 	tail.assign("\n").append(send_to).append(" K\n\n");
 
@@ -878,8 +873,8 @@ void * transmit_relay_interval(void * ptr)
 			}
 		}
 	}
-
-	wait_seconds(thread_ptr->rx_interval_time, 100);
+	
+	wait_seconds(1, 100);
 
 	// Data section
 	if(progStatus.use_header_modem || progStatus.fldigi_xmt_mode_change) {
@@ -987,7 +982,7 @@ void * transmit_interval(void * ptr)
 		if (send_to.empty()) send_to.assign("QST");
 		send_to.append(" DE ").append(progStatus.my_call);
 		for (size_t n = 0; n < send_to.length(); n++)
-			send_to[n] = toupper(send_to[n]);
+		send_to[n] = toupper(send_to[n]);
 
 		tail_bk.clear();
 		tail_bk.assign("\n").append(send_to).append(" BK\n\n");
@@ -1043,7 +1038,10 @@ void * transmit_interval(void * ptr)
 		if((fills && shift_key_flag) || !shift_key_flag) {
 
 			if(count > 0 && !transmit_stop) {
-				wait_seconds(thread_ptr->rx_interval_time, 100);
+				if(progStatus.use_txrx_interval)
+					wait_seconds(thread_ptr->rx_interval_time, 100);
+				else
+					wait_seconds(1, 100);
 			}
 
 			if(transmit_stop) {
@@ -1071,7 +1069,10 @@ void * transmit_interval(void * ptr)
 				}
 
 				if(repeat_count > 0) {
-					wait_seconds(thread_ptr->rx_interval_time, 100);
+					if(progStatus.use_txrx_interval)
+						wait_seconds(thread_ptr->rx_interval_time, 100);
+					else
+						wait_seconds(1, 100);
 				}
 
 				int headerm = progStatus.use_header_modem;
@@ -1089,10 +1090,12 @@ void * transmit_interval(void * ptr)
 				}
 
 				if(vector_header_count > 0) {
-					if(thread_ptr->mode == TX_SEGMENTED)
-						wait_seconds(thread_ptr->rx_interval_time, 100);
-					else
-						wait_seconds(1, 100);
+					if(thread_ptr->mode == TX_SEGMENTED) {
+						if(progStatus.use_txrx_interval)
+							wait_seconds(thread_ptr->rx_interval_time, 100);
+						else
+							wait_seconds(1, 100);
+					}
 				}
 
 				// Data section
@@ -1162,7 +1165,6 @@ bool send_vector_to_fldigi(TX_FLDIGI_THREAD *thread, std::string modem, std::str
 	float next_length = 0;
 	float wait_factor = 3.0;
 	float extras_time = 0.0;
-	float rsid_time = 0.0;
 	std::string send_to = tx->callto();
 
 	if(transmit_stop) return false;
@@ -1178,8 +1180,6 @@ bool send_vector_to_fldigi(TX_FLDIGI_THREAD *thread, std::string modem, std::str
 	}
 	extras_time = seconds_from_string(modem, extras, NULL);
 
-	rsid_time = (using_rsid ? rsid_duration(modem) : 0);
-
 	if(tx->amp_type() == RX_AMP)
 		send_to.clear();
 	else
@@ -1187,7 +1187,7 @@ bool send_vector_to_fldigi(TX_FLDIGI_THREAD *thread, std::string modem, std::str
 
 	send_to.append(" DE ").append(progStatus.my_call);
 	for (size_t n = 0; n < send_to.length(); n++)
-		send_to[n] = toupper(send_to[n]);
+	send_to[n] = toupper(send_to[n]);
 
 	temp.clear();
 	count = vector_data.size();
@@ -1210,7 +1210,6 @@ bool send_vector_to_fldigi(TX_FLDIGI_THREAD *thread, std::string modem, std::str
 
 	length = seconds_from_string(modem, send, &overhead);
 	length = overhead;
-	length += rsid_time;
 	length += extras_time;
 	length += 0.5;
 
@@ -1245,14 +1244,18 @@ bool send_vector_to_fldigi(TX_FLDIGI_THREAD *thread, std::string modem, std::str
 				transfer(send);
 
 				if(mode == TX_SEGMENTED) {
-					if (!wait_for_rx(length, wait_factor)) return false;
-					wait_seconds(thread->rx_interval_time, 100);
+					if (!wait_for_rx(length, wait_factor))
+						return false;
+
+					if(progStatus.use_txrx_interval)
+						wait_seconds(thread->rx_interval_time, 100);
+					else
+						wait_seconds(1, 100);
 				}
 
 				send.assign("\n");
 				temp.clear();
 				length = overhead;
-				length += rsid_time;
 				length += extras_time;
 				length += 0.5;
 			}
@@ -1384,15 +1387,19 @@ bool check_block_tx_time(std::vector<std::string> &header,
 
 	return return_value;
 }
-
+#define EXTRA_TIME 30
 /** ********************************************************
  *
  ***********************************************************/
 bool wait_for_rx(int max_wait_seconds, float factor)
 {
-	
+	int wait_time_in_seconds = 0;
+
 	if(max_wait_seconds < 1) return false;
 
+	wait_time_in_seconds = (int) (((float) max_wait_seconds) * factor);
+	wait_time_in_seconds += EXTRA_TIME;
+
 	for(int i = 0; i < 2; i++) {
 		if(transmit_stop) return false;
 
@@ -1401,9 +1408,7 @@ bool wait_for_rx(int max_wait_seconds, float factor)
 			break;
 	}
 
-	max_wait_seconds += 4; // in case RSID is enabled
-
-	for(int i = 0; i < max_wait_seconds * factor; i++) {
+	for(int i = 0; i < wait_time_in_seconds; i++) {
 		if(transmit_stop) return false;
 
 		wait_milliseconds(1000, 250);
@@ -1496,13 +1501,12 @@ TX_FLDIGI_THREAD * run_in_thread(void *(*func)(void *), int mode,
 {
 	TX_FLDIGI_THREAD *tx_thread = (TX_FLDIGI_THREAD *)0;
 	int count = 0;
-
+	size_t index_size = 0;
 	if(transmitting) return tx_thread;
 
 	tx_thread = new TX_FLDIGI_THREAD;
 
 	if(tx_thread) {
-
 		tx_thread->err_flag = false;
 		tx_thread->mode = mode;
 		tx_thread->que  = queued;
@@ -1510,6 +1514,7 @@ TX_FLDIGI_THREAD * run_in_thread(void *(*func)(void *), int mode,
 		tx_thread->rx_interval_time = cnt_rx_interval_secs->value();
 		tx_thread->tx_interval_time = floor(cnt_tx_interval_mins->value() * 60);
 		tx_thread->data = (void *) relay_data;
+		index_size = bc_modems.size();
 
 		if(relay_data)
 			tx_thread->amp_type = RX_AMP;
@@ -1517,12 +1522,13 @@ TX_FLDIGI_THREAD * run_in_thread(void *(*func)(void *), int mode,
 			tx_thread->amp_type = TX_AMP;
 
 		if(progStatus.hamcast_mode_cycle && g_event_driven && queued) {
-			if(modem_rotation_index >= bc_modems.size())
+			if(modem_rotation_index >= index_size)
 				modem_rotation_index = 0;
-
-			tx_thread->modem.assign(bc_modems[modem_rotation_index]);
+			g_modem.assign(bc_modems[modem_rotation_index]);
+			//LOG_DEBUG("%s %d", g_modem.c_str(), modem_rotation_index);
+			tx_thread->modem.assign(g_modem);
 			modem_rotation_index++;
-
+			modem_rotation_index &= 0x3;
 		} else {
 			g_modem.assign(cbo_modes->value());
 			tx_thread->modem.assign(g_modem);
@@ -1575,8 +1581,8 @@ void * run_in_thread_destroy(TX_FLDIGI_THREAD *tx_thread, int level,
 		tx_thread_running_count--;
 
 	{
-		static int value = TX_BUTTON;
-		Fl::awake(deactivate_button, (void *) &value);
+	static int value = TX_BUTTON;
+	Fl::awake(deactivate_button, (void *) &value);
 	}
 
 	pthread_mutex_lock(&tx_thread->mutex);
@@ -1603,17 +1609,15 @@ void * run_in_thread_destroy(TX_FLDIGI_THREAD *tx_thread, int level,
 
 	delete tx_thread;
 
-
 	Fl::awake(set_relay_button_label, (void *)0);
 	Fl::awake(set_button_to_xmit, (void *)0);
 	Fl::awake(clear_missing, (void *)0);
 
-
 	{
-		static int value = TX_BUTTON;
-		Fl::awake(activate_button, (void *)&value);
-		static int value2 = TX_ALL_BUTTON;
-		Fl::awake(activate_button, (void *)&value2);
+	static int value = TX_BUTTON;
+	Fl::awake(activate_button, (void *)&value);
+	static int value2 = TX_ALL_BUTTON;
+	Fl::awake(activate_button, (void *)&value2);
 	}
 
 	wait_for_rx(5);
